/**
 * @file    net-mind.c
 * @author  Ikkei Shimomura 
 * @date    2005/05/16
 *
 * $Id: net-mine.c,v 1.4 2005/05/16 10:39:21 tea Exp tea $
 */
#include "net-mine.h"

static MineBoard net_mine_board;

MineBoard *net_mine_board_init(MineBoard *board, size_t cols, size_t rows)
{
    assert(cols < MAX_COLS);
    assert(rows < MAX_ROWS);
    
    BOARD_LOOP_INIT_VARS(board_col, board_row);

    BOARD_CHECK(board);
    if (board->screen != NULL)
        net_mine_board_delete(board);

    board->cols = cols;
    board->rows = rows;
    board->count = 0;
    board->screen = (net_mine_t *) malloc(sizeof(net_mine_t) * cols * rows);

    BOARD_LOOP_ITER(board, board_col, board_row) {
        BOARD_ITEM(board, board_col, board_row) = (net_mine_t) 0x00;
    }

    return board;
}

MineBoard *net_mine_board_delete(MineBoard *board)
{
    BOARD_CHECK(board);

    if (board->screen != NULL) {
        free(board->screen);
        board->screen = NULL;
    }
    board->cols = board->rows = board->count = 0;

    return board;
}

/* private function */
net_mine_t
net_mine_board_count_bomb(MineBoard *board, size_t col, size_t row)
{
    size_t num = 0;

    /* TODO Make the macro for TILE_LOOP_AROUND ? */
    num += HAS_BOMB(board, col-1, row-1) ? 1 : 0;
    num += HAS_BOMB(board, col+0, row-1) ? 1 : 0;
    num += HAS_BOMB(board, col+1, row-1) ? 1 : 0;
    num += HAS_BOMB(board, col-1, row+0) ? 1 : 0;
    num += HAS_BOMB(board, col+1, row+0) ? 1 : 0;
    num += HAS_BOMB(board, col-1, row+1) ? 1 : 0;
    num += HAS_BOMB(board, col+0, row+1) ? 1 : 0;
    num += HAS_BOMB(board, col+1, row+1) ? 1 : 0;

    assert(0 <= num && num <= 8);

    return (net_mine_t) num;
}

/**
 * @macro CHECK_ARGS_AND_INIT
 * @param MineBoard*    board
 * @param MinePacket*   packet
 * @return MinePacket*
 * 
 * Checking MineBoard and MinePacket are not null,
 * If one of them null pointer then return NULL else itinialize MinePacket.
 */
#define CHECK_ARGS_AND_INIT(_board,_packet) \
    { BOARD_CHECK(_board); PACKET_CHECK(_packet); PACKET_CLEAR(_packet); }

MinePacket *net_mine_board_open(MineBoard *board, MinePacket *packet,
                                size_t col, size_t row)
{
    CHECK_ARGS_AND_INIT(board, packet);

    if (ON_BOARD(board, col, row)) {
        if (IS_NOT_OPENED(board, col, row)) {
            BOARD_ITEM(board, col, row) |= OPEN;

            /* TODO open */
        }

        PACKET_SET(packet, board, col, row);
    }

    return packet;
}

MinePacket *net_mine_board_flag(MineBoard *board, MinePacket *packet,
                                size_t col, size_t row)
{
    CHECK_ARGS_AND_INIT(board, packet);

    if (ON_BOARD(board, col, row)) {
        BOARD_ITEM(board, col, row) ^= FLAG;
        PACKET_SET(packet, board, col, row);
    }

    return packet;
}
#undef CHECK_ARGS_AND_INIT /* no use this macro anymore */

DEFINE_COMMAND(net_mine_cmd_init, INIT);
DEFINE_COMMAND(net_mine_cmd_quit, QUIT);
DEFINE_COMMAND_EX(net_mine_cmd_open, OPEN);
DEFINE_COMMAND_EX(net_mine_cmd_flag, FLAG);
